module soc_system # (
    parameter NUM_OF_CHANNEL = 2
 )(
    // FPGA clock and reset
    input  wire          fpga_clk_100,
    input  wire          fpga_reset_n, // dev_clrn

    input                fpga_clk_50, 
    input                refclk, // clk_enet_fpga_p

    // ETHERNET 1
    output           eneta_mdc,      
    inout            eneta_mdio,     
    input            eneta_rx_p,     
    output           eneta_tx_p,     
    input           eneta_intn,     //FPGA_IO2 -> ENETA_INTn 
    output           eneta_resetn,


    // ETHERNET 2

    output           enetb_mdc,      
	inout            enetb_mdio,     
	input            enetb_rx_p,     
	output           enetb_tx_p,     
	input           enetb_intn,    //FPGA_IO3 -> ENETB_INTn 
	output           enetb_resetn,
    
    
  // HPS memory controller ports
  // DDR4 single rank -2133
  input  wire          emif_ref_clk,
  output wire          hps_memory_mem_act_n,
  output wire          hps_memory_mem_bg,
  output wire          hps_memory_mem_par,
  input  wire          hps_memory_mem_alert_n,
  inout  wire [ 3:0]   hps_memory_mem_dbi_n,
  output wire [16:0]   hps_memory_mem_a,
  output wire [ 1:0]   hps_memory_mem_ba,
  output wire          hps_memory_mem_ck,
  output wire          hps_memory_mem_ck_n,
  output wire          hps_memory_mem_cke,
  output wire          hps_memory_mem_cs_n,
  output wire          hps_memory_mem_reset_n,
  inout  wire [31:0]   hps_memory_mem_dq,
  inout  wire [ 3:0]   hps_memory_mem_dqs,
  inout  wire [ 3:0]   hps_memory_mem_dqs_n,
  output wire          hps_memory_mem_odt,
  input  wire          hps_memory_oct_rzqin,
  // HPS peripherals
  output wire          hps_emac0_TX_CLK,
  output wire          hps_emac0_TXD0,
  output wire          hps_emac0_TXD1,
  output wire          hps_emac0_TXD2,
  output wire          hps_emac0_TXD3,
  input  wire          hps_emac0_RXD0,
  inout  wire          hps_emac0_MDIO,
  output wire          hps_emac0_MDC,
  input  wire          hps_emac0_RX_CTL,
  output wire          hps_emac0_TX_CTL,
  input  wire          hps_emac0_RX_CLK,
  input  wire          hps_emac0_RXD1,
  input  wire          hps_emac0_RXD2,
  input  wire          hps_emac0_RXD3,
  inout  wire          hps_usb0_D0,
  inout  wire          hps_usb0_D1,
  inout  wire          hps_usb0_D2,
  inout  wire          hps_usb0_D3,
  inout  wire          hps_usb0_D4,
  inout  wire          hps_usb0_D5,
  inout  wire          hps_usb0_D6,
  inout  wire          hps_usb0_D7,
  input  wire          hps_usb0_CLK,
  output wire          hps_usb0_STP,
  input  wire          hps_usb0_DIR,
  input  wire          hps_usb0_NXT,
  output wire          hps_spim1_CLK,
  output wire          hps_spim1_MOSI,
  input  wire          hps_spim1_MISO,
  output wire          hps_spim1_SS0_N,
  output wire          hps_spim1_SS1_N,
  input  wire          hps_uart1_RX,
  output wire          hps_uart1_TX,
  inout  wire          hps_i2c1_SDA,
  inout  wire          hps_i2c1_SCL,
  inout  wire          hps_sdio_CMD,
  output wire          hps_sdio_CLK,
  inout  wire          hps_sdio_D0,
  inout  wire          hps_sdio_D1,
  inout  wire          hps_sdio_D2,
  inout  wire          hps_sdio_D3,
  inout  wire          hps_sdio_D4,
  inout  wire          hps_sdio_D5,
  inout  wire          hps_sdio_D6,
  inout  wire          hps_sdio_D7
);

// internal wires and registers declaration
wire [27:0] a10_stm_hw_events;
wire [31:0] a10_hps_f2h_irq0;
wire [31:0] a10_hps_f2h_irq1;
wire        tse_resetn;
// connection of internal logics
assign a10_stm_hw_events = '0;
assign a10_hps_f2h_irq0  = '0;
assign a10_hps_f2h_irq1  = '0;


localparam ADDRESS_WIDTH = 21;
localparam ID_WIDTH      = 4;
localparam DATA_BYTES    = 4;

//------------------------------------------
//------------- INTERFACES -----------------
//------------------------------------------
axi4_mm_if #(
    .DATA_BYTES      (DATA_BYTES),
    .ADDRESS_WIDTH   (ADDRESS_WIDTH),
    .ID_WIDTH        (ID_WIDTH)
) a10_hps_h2f_lw_axi_master (
    .aclk            (fpga_clk_100),
    .areset_n        (fpga_reset_n)
);

logic        s_axil_awready;
logic        s_axil_awvalid;
logic [20:0] s_axil_awaddr;
logic [ 2:0] s_axil_awprot;

logic        s_axil_wready;
logic        s_axil_wvalid;
logic [31:0] s_axil_wdata;
logic [ 3:0] s_axil_wstrb;

logic        s_axil_bready;
logic        s_axil_bvalid;
logic [ 1:0] s_axil_bresp;

logic        s_axil_arready;
logic        s_axil_arvalid;
logic [20:0] s_axil_araddr;
logic [ 2:0] s_axil_arprot;

logic        s_axil_rready;
logic        s_axil_rvalid;
logic        s_axil_rlast;
logic [31:0] s_axil_rdata;
logic [ 1:0] s_axil_rresp;

reg [ 3:0] w_id;
reg [ 3:0] r_id;

// Interconnect of AXI-MM Master & Slave IF
always_ff @(posedge fpga_clk_100 or negedge fpga_reset_n) begin
    if (!fpga_reset_n) begin 
        w_id <= '0;
        r_id <= '0;
    end
    else begin
        if (a10_hps_h2f_lw_axi_master.awvalid) begin
            w_id <= a10_hps_h2f_lw_axi_master.awid;
        end

        if (a10_hps_h2f_lw_axi_master.arvalid) begin
            r_id <= a10_hps_h2f_lw_axi_master.arid;
        end 
    end
end



assign s_axil_awaddr                        = a10_hps_h2f_lw_axi_master.awaddr;
assign s_axil_awprot                        = a10_hps_h2f_lw_axi_master.awprot;
assign s_axil_awvalid                       = a10_hps_h2f_lw_axi_master.awvalid;
assign a10_hps_h2f_lw_axi_master.awready    = s_axil_awready;

assign s_axil_wdata[7:0]                    = a10_hps_h2f_lw_axi_master.wdata[0];
assign s_axil_wdata[15:8]                   = a10_hps_h2f_lw_axi_master.wdata[1];
assign s_axil_wdata[23:16]                  = a10_hps_h2f_lw_axi_master.wdata[2];
assign s_axil_wdata[31:24]                  = a10_hps_h2f_lw_axi_master.wdata[3];
assign s_axil_wstrb                         = a10_hps_h2f_lw_axi_master.wstrb;
assign s_axil_wvalid                        = a10_hps_h2f_lw_axi_master.wvalid;
assign a10_hps_h2f_lw_axi_master.wready     = s_axil_wready;

assign a10_hps_h2f_lw_axi_master.bid        = w_id;
assign a10_hps_h2f_lw_axi_master.bresp      = s_axil_bresp;
assign a10_hps_h2f_lw_axi_master.buser      = '0;
assign a10_hps_h2f_lw_axi_master.bvalid     = s_axil_bvalid;
assign s_axil_bready                        = a10_hps_h2f_lw_axi_master.bready;

assign s_axil_araddr                        = a10_hps_h2f_lw_axi_master.araddr;
assign s_axil_arprot                        = a10_hps_h2f_lw_axi_master.arprot;
assign s_axil_arvalid                       = a10_hps_h2f_lw_axi_master.arvalid;
assign a10_hps_h2f_lw_axi_master.arready    = s_axil_arready;

assign a10_hps_h2f_lw_axi_master.rid        = r_id;
assign a10_hps_h2f_lw_axi_master.rdata[0]   = s_axil_rdata[7:0];
assign a10_hps_h2f_lw_axi_master.rdata[1]   = s_axil_rdata[15:8];
assign a10_hps_h2f_lw_axi_master.rdata[2]   = s_axil_rdata[23:16];
assign a10_hps_h2f_lw_axi_master.rdata[3]   = s_axil_rdata[31:24];
assign a10_hps_h2f_lw_axi_master.rresp      = s_axil_rresp;
assign a10_hps_h2f_lw_axi_master.rlast      = s_axil_rlast;
assign a10_hps_h2f_lw_axi_master.ruser      = '0;
assign a10_hps_h2f_lw_axi_master.rvalid     = s_axil_rvalid;
assign s_axil_rready                        = a10_hps_h2f_lw_axi_master.rready;

assign s_axil_rlast                         = (s_axil_rvalid & s_axil_rready);


modul_studenta modul_studenta_u (
    .rst             ( ~fpga_reset_n  ),
    .clk             ( fpga_clk_100   ),

    .s_axil_awready  ( s_axil_awready ),
    .s_axil_awvalid  ( s_axil_awvalid ),
    .s_axil_awaddr   ( s_axil_awaddr  ),
    .s_axil_awprot   ( s_axil_awprot  ),
    .s_axil_wready   ( s_axil_wready  ),
    .s_axil_wvalid   ( s_axil_wvalid  ),
    .s_axil_wdata    ( s_axil_wdata   ),
    .s_axil_wstrb    ( s_axil_wstrb   ),
    .s_axil_bready   ( s_axil_bready  ),
    .s_axil_bvalid   ( s_axil_bvalid  ),
    .s_axil_bresp    ( s_axil_bresp   ),
    .s_axil_arready  ( s_axil_arready ),
    .s_axil_arvalid  ( s_axil_arvalid ),
    .s_axil_araddr   ( s_axil_araddr  ),
    .s_axil_arprot   ( s_axil_arprot  ),
    .s_axil_rready   ( s_axil_rready  ),
    .s_axil_rvalid   ( s_axil_rvalid  ),
    .s_axil_rdata    ( s_axil_rdata   ),
    .s_axil_rresp    ( s_axil_rresp   ),

    .LED             (  )
);



//------------------------------------------
//------------- ETHERNET -----------------
//------------------------------------------1

wire    eneta_mdio_oen;
wire    eneta_mdio_out;

assign eneta_mdio = eneta_mdio_oen == 0 ? eneta_mdio_out : 1'bz;

wire    enetb_mdio_oen;
wire    enetb_mdio_out;

assign enetb_mdio = enetb_mdio_oen == 0 ? enetb_mdio_out : 1'bz;

  localparam RESET_SOURCES_COUNT = 2;
  
  reg [(RESET_SOURCES_COUNT - 1):0]   resetn_sources;
  wire            global_resetn;
  wire            reset_phy_clk_n_from_the_ddr3_top;
  wire            tx_serial_clk;


always @ (*) begin
    resetn_sources[(RESET_SOURCES_COUNT - 1)]   <=  fpga_reset_n;
    //resetn_sources[(RESET_SOURCES_COUNT - 2)]   <=  reset_phy_clk_n_from_the_ddr3_top;
    resetn_sources[(RESET_SOURCES_COUNT - 2)]   <=  1'b1; // don't need another reset source here
end

global_reset_generator 
#(
    .RESET_SOURCES_WIDTH  (RESET_SOURCES_COUNT),
    .RESET_COUNTER_WIDTH  (16)
) global_reset_generator_inst
(
    .clk            (fpga_clk_50),
    .resetn_sources (resetn_sources),
    .global_resetn  (global_resetn),
    .pll_resetn     ()
);


assign eneta_resetn = global_resetn;
assign enetb_resetn = global_resetn;


//------------------------------------------
//------------- HPS WRAPPER ----------------
//------------------------------------------
hps_wrapper u_hps_wrapper (
    // FPGA -> HPS Reset and interrupt
    .a10_hps_f2h_cold_reset_n       (fpga_reset_n),
    .a10_hps_f2h_debug_reset_n      (fpga_reset_n),
    .a10_hps_f2h_warm_reset_n       (fpga_reset_n),
    .a10_hps_f2h_irq0               (a10_hps_f2h_irq0),
  	.a10_hps_f2h_irq1               (a10_hps_f2h_irq1),
    .a10_hps_f2h_stm_hw_events      (a10_stm_hw_events),

    // HPS -> FPGA AXI MASTER
    .a10_hps_h2f_axi_master_clk     (1'b0),
    .a10_hps_h2f_axi_master_reset_n (1'b0),
    .a10_hps_h2f_axi_master         (),

    // HPS -> FPGA AXI MASTER LIGHTWEIGHT
    .a10_hps_h2f_lw_axi_master_clk      (fpga_clk_100),
    .a10_hps_h2f_lw_axi_master_reset_n  (fpga_reset_n),
    .a10_hps_h2f_lw_axi_master          (a10_hps_h2f_lw_axi_master),

    // FPGA -> HPS AXI SLAVE
    .a10_hps_f2h_axi_slave_clk      (1'b0),
    .a10_hps_f2h_axi_slave_reset_n  (1'b0),
    .a10_hps_f2h_axi_slave          (),


    // DDR4 single rank -2133 device
    .emif_ref_clk                   (emif_ref_clk),
    .emif_reset_n                   (fpga_reset_n),
    .hps_memory_mem_act_n           (hps_memory_mem_act_n),
    .hps_memory_mem_bg              (hps_memory_mem_bg),
    .hps_memory_mem_par             (hps_memory_mem_par),
    .hps_memory_mem_alert_n         (hps_memory_mem_alert_n),
    .hps_memory_mem_dbi_n           (hps_memory_mem_dbi_n),
    .hps_memory_mem_a               (hps_memory_mem_a),
    .hps_memory_mem_ba              (hps_memory_mem_ba),
    .hps_memory_mem_ck              (hps_memory_mem_ck),
    .hps_memory_mem_ck_n            (hps_memory_mem_ck_n),
    .hps_memory_mem_cke             (hps_memory_mem_cke),
    .hps_memory_mem_cs_n            (hps_memory_mem_cs_n),
    .hps_memory_mem_reset_n         (hps_memory_mem_reset_n),
    .hps_memory_mem_dq              (hps_memory_mem_dq),
    .hps_memory_mem_dqs             (hps_memory_mem_dqs),
    .hps_memory_mem_dqs_n           (hps_memory_mem_dqs_n),
    .hps_memory_mem_odt             (hps_memory_mem_odt),
    .hps_memory_oct_rzqin           (hps_memory_oct_rzqin),

    // HPS peripherals
    .a10_hps_phery_emac0_TX_CLK     (hps_emac0_TX_CLK),
    .a10_hps_phery_emac0_TXD0       (hps_emac0_TXD0),
    .a10_hps_phery_emac0_TXD1       (hps_emac0_TXD1),
    .a10_hps_phery_emac0_TXD2       (hps_emac0_TXD2),
    .a10_hps_phery_emac0_TXD3       (hps_emac0_TXD3),
    .a10_hps_phery_emac0_RX_CTL     (hps_emac0_RX_CTL),
    .a10_hps_phery_emac0_TX_CTL     (hps_emac0_TX_CTL),
    .a10_hps_phery_emac0_RX_CLK     (hps_emac0_RX_CLK),
    .a10_hps_phery_emac0_RXD0       (hps_emac0_RXD0),
    .a10_hps_phery_emac0_RXD1       (hps_emac0_RXD1),
    .a10_hps_phery_emac0_RXD2       (hps_emac0_RXD2),
    .a10_hps_phery_emac0_RXD3       (hps_emac0_RXD3),
    .a10_hps_phery_emac0_MDIO       (hps_emac0_MDIO),
    .a10_hps_phery_emac0_MDC        (hps_emac0_MDC),
    .a10_hps_phery_sdmmc_CMD        (hps_sdio_CMD),
    .a10_hps_phery_sdmmc_D0         (hps_sdio_D0),
    .a10_hps_phery_sdmmc_D1         (hps_sdio_D1),
    .a10_hps_phery_sdmmc_D2         (hps_sdio_D2),
    .a10_hps_phery_sdmmc_D3         (hps_sdio_D3),
    .a10_hps_phery_sdmmc_D4         (hps_sdio_D4),
    .a10_hps_phery_sdmmc_D5         (hps_sdio_D5),
    .a10_hps_phery_sdmmc_D6         (hps_sdio_D6),
    .a10_hps_phery_sdmmc_D7         (hps_sdio_D7),
    .a10_hps_phery_sdmmc_CCLK       (hps_sdio_CLK),
    .a10_hps_phery_usb0_DATA0       (hps_usb0_D0),
    .a10_hps_phery_usb0_DATA1       (hps_usb0_D1),
    .a10_hps_phery_usb0_DATA2       (hps_usb0_D2),
    .a10_hps_phery_usb0_DATA3       (hps_usb0_D3),
    .a10_hps_phery_usb0_DATA4       (hps_usb0_D4),
    .a10_hps_phery_usb0_DATA5       (hps_usb0_D5),
    .a10_hps_phery_usb0_DATA6       (hps_usb0_D6),
    .a10_hps_phery_usb0_DATA7       (hps_usb0_D7),
    .a10_hps_phery_usb0_CLK         (hps_usb0_CLK),
    .a10_hps_phery_usb0_STP         (hps_usb0_STP),
    .a10_hps_phery_usb0_DIR         (hps_usb0_DIR),
    .a10_hps_phery_usb0_NXT         (hps_usb0_NXT),
    .a10_hps_phery_spim1_CLK        (hps_spim1_CLK),
    .a10_hps_phery_spim1_MOSI       (hps_spim1_MOSI),
    .a10_hps_phery_spim1_MISO       (hps_spim1_MISO),
    .a10_hps_phery_spim1_SS0_N      (hps_spim1_SS0_N),
    .a10_hps_phery_spim1_SS1_N      (hps_spim1_SS1_N),
    .a10_hps_phery_uart1_RX         (hps_uart1_RX),
    .a10_hps_phery_uart1_TX         (hps_uart1_TX),
    .a10_hps_phery_i2c1_SDA         (hps_i2c1_SDA),
    .a10_hps_phery_i2c1_SCL         (hps_i2c1_SCL),
	 
	.clk_125m_clk										(refclk),
	.reset_125m_reset_n								(global_resetn), 
	
	// Triple Speed Ethernet
	.eth_tse_0_mac_mdio_connection_mdc			(eneta_mdc),
	.eth_tse_0_mac_mdio_connection_mdio_in		(eneta_mdio),
	.eth_tse_0_mac_mdio_connection_mdio_out	(eneta_mdio_out),
	.eth_tse_0_mac_mdio_connection_mdio_oen	(eneta_mdio_oen),
	.eth_tse_0_serial_connection_rxp				(eneta_rx_p),
	.eth_tse_0_serial_connection_txp				(eneta_tx_p),
	.eth_tse_0_status_led_connection_crs		(),
	.eth_tse_0_status_led_connection_link		(),
	.eth_tse_0_status_led_connection_panel_link (),
	.eth_tse_0_status_led_connection_col		(),
	.eth_tse_0_status_led_connection_an			(),
	.eth_tse_0_status_led_connection_char_err	(),
	.eth_tse_0_status_led_connection_disp_err	(),
	
	.xcvr_atx_pll_a10_0_pll_cal_busy_pll_cal_busy (), 
	.xcvr_atx_pll_a10_0_pll_locked_pll_locked      (),

    .eth_tse_1_mac_mdio_connection_mdc			(enetb_mdc),
	.eth_tse_1_mac_mdio_connection_mdio_in		(enetb_mdio),
	.eth_tse_1_mac_mdio_connection_mdio_out	(enetb_mdio_out),
	.eth_tse_1_mac_mdio_connection_mdio_oen	(enetb_mdio_oen),
	.eth_tse_1_serial_connection_rxp				(enetb_rx_p),
	.eth_tse_1_serial_connection_txp				(enetb_tx_p),
	.eth_tse_1_status_led_connection_crs		(),
	.eth_tse_1_status_led_connection_link		(),
	.eth_tse_1_status_led_connection_panel_link (),
	.eth_tse_1_status_led_connection_col		(),
	.eth_tse_1_status_led_connection_an			(),
	.eth_tse_1_status_led_connection_char_err	(),
	.eth_tse_1_status_led_connection_disp_err	()

);



endmodule
